// main.cpp
#include <ctime>
#include <iostream>

const char scszSSCMExecutable[] = "\"C:\\Program Files\\Seapine\\Surround SCM\\sscm.exe\"";
const char cszDate_Today[] = "Today";
const char cszDate_Yesterday[] = "Yesterday";
const char cszDate_ThisWeek[] = "ThisWeek";
const char cszDate_LastWeek[] = "LastWeek";
const char cszDate_ThisMonth[] = "ThisMonth";
const char cszDate_LastMonth[] = "LastMonth";
const char cszDate_ThisYear[] = "ThisYear";
const char cszDate_LastYear[] = "LastYear";
const char cszDate_XMonths[] = "Months";
const char cszDate_XWeeks[] = "Weeks";
const char cszDate_XDays[] = "Days";

const double cdSecondsInADay = 86400;

using namespace std;

void subtractMonths(tm &p_tm, int p_i);

int main(int p_argc, char *p_argv[])
{
   if (p_argc < 2)
   {
      cout << "\nYou must enter the path to the sscm executable (or 'default')\n\
\tex: \"\\\"C:\\Program Files\\Seapine\\Surround SCM\\sscm.exe\\\"\"\n\
as the first cmd-line parameter, then begin passing the 'rh'\n\
parameters to generate a Surround History report.\n\n\
You can use the following keywords with -d instead of having to hard-code\n\
a date range when using the CLI directly.\n\n";
      cout << "\t" << cszDate_Today << " :  -dToday \n";
      cout << "\t" << cszDate_Yesterday << " :  -dYesterday \n";
      cout << "\t" << cszDate_ThisWeek << " :  -dThisWeek \n";
      cout << "\t" << cszDate_LastWeek << " :  -dLastWeek \n";
      cout << "\t" << cszDate_ThisMonth << " :  -dThisMonth \n";
      cout << "\t" << cszDate_LastMonth << " :  -dLastMonth \n";
      cout << "\t" << cszDate_ThisYear << " :  -dThisYear \n";
      cout << "\t" << cszDate_LastYear << " :  -dLastYear \n";
      cout << "\tlast x" << cszDate_XMonths << " :  -d12Months (1 will give you ThisMonth) \n";
      cout << "\tlast x" << cszDate_XWeeks << " :  -d52Weeks (1 will give you ThisWeek) \n";
      cout << "\tlast x" << cszDate_XDays << " :  -d7Days (1 will give you Today) \n";
      return (0);
   }

   // Did they give us a -d?
   int iDashDPos = -1;
   for (int i = 2; i < p_argc; ++i)
   {
      if (strlen(p_argv[i]) > 1 && p_argv[i][0] == '-' && p_argv[i][1] == 'd')
      {
         if (iDashDPos = -1)  { iDashDPos = i; }
         else                 { std:cout << "ERROR: You have specified more than one -d option!"; return (0); }
      }
   }

   char *pszDateRange = 0;
   if (iDashDPos != -1)
   {
      // First, is it a token or just raw date/time information. Compare the last 
      // character to make sure there is a difference between raw and keyword
      if (!(p_argv[iDashDPos][strlen(p_argv[iDashDPos])-1] >= 48 && p_argv[iDashDPos][strlen(p_argv[iDashDPos])-1] <= 57))
      {
         // Setup Date/Time data
         time_t tToday = time(0);
         struct tm tmToday = *localtime(&tToday);
         struct tm tmStartDate;
         struct tm tmEndDate;

         bool fValid = false;
         char *pszKeyword = p_argv[iDashDPos];
         ++++pszKeyword; // move past '-d'
         char szToCompare[128];

         if (stricmp(pszKeyword, cszDate_Today) == 0)
         {
            tmStartDate = tmToday;
            tmEndDate = tmToday;
            fValid = true;
         }
         else if (stricmp(pszKeyword, cszDate_Yesterday) == 0)
         {
            time_t tYesterday = tToday - cdSecondsInADay; // Take off one day's worth of seconds.
            tmStartDate = *localtime(&tYesterday);
            tmEndDate = *localtime(&tYesterday);
            fValid = true;
         }
         else if (stricmp(pszKeyword, cszDate_ThisWeek) == 0)
         {
            time_t tStart = tToday - (tmToday.tm_wday * cdSecondsInADay); // Take off one X day's worth of seconds, depending on where we are in the week.
            tmStartDate = *localtime(&tStart);
            tmEndDate = tmToday;
            fValid = true;
         }
         else if (stricmp(pszKeyword, cszDate_ThisMonth) == 0)
         {
            tmStartDate = tmToday;
            tmStartDate.tm_mday = 1;
            tmEndDate = tmToday;
            fValid = true;
         }
         else if (stricmp(pszKeyword, cszDate_LastMonth) == 0)
         {
            time_t tPrevMonth = tToday - (tmToday.tm_mday * cdSecondsInADay) - cdSecondsInADay; // Take off enough seconds to get to last month.
            tmStartDate = *localtime(&tPrevMonth);
            tmStartDate.tm_mday = 1;
            tmEndDate = *localtime(&tPrevMonth);
            fValid = true;
         }
         else if (stricmp(pszKeyword, cszDate_ThisYear) == 0)
         {
            tmStartDate = tmToday;
            tmStartDate.tm_mon = 0;
            tmStartDate.tm_mday = 1;
            tmEndDate = tmToday;
            fValid = true;
         }
         else if (stricmp(pszKeyword, cszDate_LastYear) == 0)
         {
            tmStartDate = tmToday;
            tmStartDate.tm_year -= 1;
            tmStartDate.tm_mon = 0;
            tmStartDate.tm_mday = 1;

            tmEndDate = tmToday;
            tmEndDate.tm_year -= 1;
            tmEndDate.tm_mon = 11;
            tmEndDate.tm_mday = 31;
            fValid = true;
         }
         else if (p_argv[iDashDPos][2] >= 48 && p_argv[iDashDPos][2] <= 57)
         {
            char *pszTemp = p_argv[iDashDPos];
            ++++pszTemp; // move past '-d'
            string strNum = "";
            for (; *pszTemp >= 48 && *pszTemp <= 57; ++pszTemp)
               strNum += *pszTemp;

            int iNum = atoi(strNum.data());
            iNum = (iNum <= 0) ? 0 : iNum-1;

            if (stricmp(pszTemp, cszDate_XMonths) == 0)
            {
               tmStartDate = tmToday;
               subtractMonths( tmStartDate, iNum);
               tmStartDate.tm_mday = 1; // Always start at the first of the month.

               tmEndDate = tmToday;
               fValid = true;
            }
            else if (stricmp(pszTemp, cszDate_XWeeks) == 0)
            {
               time_t tStart = tToday - (iNum * 7 * cdSecondsInADay); // Get to the week we're interested in.
               tStart = tStart - (tmToday.tm_wday * cdSecondsInADay); // Get to the start of that week.
               tmStartDate = *localtime(&tStart);
               tmEndDate = tmToday;
               fValid = true;
            }
            else if (stricmp(pszTemp, cszDate_XDays) == 0)
            {
               time_t tStart = tToday - (iNum * cdSecondsInADay); // Take off X days worth of seconds.
               tmStartDate = *localtime(&tStart);
               tmEndDate = tmToday;
               fValid = true;
            }
         }

         if (fValid)
         {
            char szStartDate[17] = {0};
            strftime( szStartDate, 16, "%Y%m%d", &tmStartDate);
            strcat( szStartDate, "00:00:00");

            char szEndDate[17] = {0};
            strftime( szEndDate, 16, "%Y%m%d", &tmEndDate);
            strcat( szEndDate, "23:59:59");

            pszDateRange = new char[strlen(szStartDate) + strlen(szEndDate) + 2];
            strcpy( pszDateRange, szStartDate);
            strcat( pszDateRange, "-");
            strcat( pszDateRange, szEndDate);
         }
      }

   }

   char szCmdLine[4096] = { 0 };

   if (strcmp(p_argv[1], "default") == 0)
      strcpy( szCmdLine, scszSSCMExecutable);
   else
      strcpy( szCmdLine, p_argv[1]);

   strcat( szCmdLine, " rh ");
   for (int i2 = 2; i2 < p_argc; ++i2)
   {
      if (i2 == iDashDPos && pszDateRange)
      {
         strcat( szCmdLine, " -d");
         strcat( szCmdLine, pszDateRange);
      }
      else
         strcat( szCmdLine, p_argv[i2]);
      strcat( szCmdLine, " ");
   }

//   std::cout << sCmdLine;
   int iResult = system( szCmdLine);
   if (pszDateRange) { delete [] pszDateRange; }

   return (iResult);
}

void subtractMonths(tm &p_tm, int p_i)
{
   if (p_tm.tm_mon >= p_i)
      p_tm.tm_mon -= p_i;
   else
   {
      int iNumYr = p_i/12;
      p_tm.tm_year -= iNumYr;

      int iMonMod = p_i % 12;
      if (p_tm.tm_mon >= iMonMod)
         p_tm.tm_mon -= iMonMod;
      else
      {
         p_tm.tm_mon = 12 + p_tm.tm_mon - iMonMod;
         p_tm.tm_year -= 1;
      }
   }
}

